/*
 * mdimmer.c
 *
 * Created: 2014-05-28 21:35:59
 *  Author: ds
 */ 


#define F_CPU 9600000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
//#include "wdog.h"

void HardInit(void);


struct {
	volatile char power;
	volatile char idle;
	volatile char phase;
	char power_steps;
	char idle_steps;
	char soft;
} system;



//-----------------------------------------------------
int main(void)
{
	char i;
	char temp;
	HardInit();
	
	//------------------------------------------------------	
	system.power = 0;
	system.idle = 0x80;
	system.soft = 1;
	i = 0;
	
	LED_SET;	
	_delay_ms(200);
	LED_OFF;
	_delay_ms(200);
	
	//------------------------------------------------------
	//start timer
	//OCR0A = 0xFF;			//define TOP value
	//OCR0B = 0x80;			//define COMP value
	//TCCR0A = (1<<WGM01 | 1<<WGM00);	 //FAST PWM

	//if (system.mode == 0)	//pwm mode	- two led blinks
	//	TCCR0B = (1<<WGM02 | 1<<CS01 | 1<<CS00);	 //start, prescale = 64
	//else
	//	TCCR0B = (1<<WGM02 | 1<<CS00);	 //start, prescale = 0
	
	//TIMSK0 = (1<<TOIE0 | 1<<OCIE0B);
		
	//------------------------------------------------------
    while(1)
    {
		//--------------------------------------------------
		//get power value
		ADMUX = (1<<ADLAR | 1<<MUX0);
		ADCSRA |= (1<<ADSC);
		while (ADCSRA & (1<<ADSC));
		temp =  (ADCH / 16);
		temp *= 2;	
		system.power = (temp + 1);		
		
		//--------------------------------------------------
		//get idle value
		ADMUX = (1<<ADLAR | 1<<MUX1);
		ADCSRA |= (1<<ADSC);
		while (ADCSRA & (1<<ADSC));
		temp =  (ADCH / 4);
		temp *= 2;
		//temp += 1;
		system.idle = (temp + 1);
		
		//-------------------------------------------------
		//soft start system
		/*if (system.soft)
		{
			i++;
			if (i > 20) 
			{
				i = 0;
				if (system.idle < temp) 
					system.idle++;
				else
					system.soft = 0;	
			}
		}
		else
		{
			system.idle = temp;
		}*/
	
		//-------------------------------------------------	
		
		_delay_ms(100);
    }
}


//-----------------------------------------------------
void HardInit(void)
{
	PORTB = (1<<LED_PIN | 1<<RST_PIN);
	DDRB = (1<<DRV_PIN | 1<<LED_PIN);

	ACSR = (1<<ACD);		//turn off comparator
	ADMUX = (1<<ADLAR);		//ADC init
	ADCSRA = (1<<ADEN | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);
	ADCSRA = (1<<ADEN | 1<<ADSC | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);

	PCMSK = (1<<PCINT0);
	GIMSK = (1<<PCIE);		//Enable PIN CHANGE int
	
	sei();
}

//-----------------------------------------------------
SIGNAL(PCINT0_vect)
{
	if (PH_GET)				//wave start
	{
		//---------------------------------------------------------
		if (system.phase == 0)		//idle time
		{
			LED_OFF;
			if (system.idle_steps < system.idle)
			{
				system.idle_steps++;
			}
			else
			{
				if (system.idle < 0x7F)
				{
					system.power_steps = 0;			//init new cycle
					system.phase = 1;
				}
				else
				{
					system.idle_steps = 0;
				}
			}
		}
		//---------------------------------------------------------
		else						//power time
		{
			LED_SET;
			if (system.power_steps < system.power)
			{
				system.power_steps++;
			}
			else
			{
				if (system.idle > 1)
				{
					system.idle_steps = 0;			//init new cycle
					system.phase = 0;					
				}
				else
					system.power_steps = 0;				
			}
		}
	}
	else					//end of wave
	{
		_delay_ms(1);
		if (system.phase == 0)
			DRV_OFF;
		else
			DRV_SET;
	}
}

//-----------------------------------------------------
SIGNAL(TIM0_OVF_vect)
{
	LED_SET;				
}

//-----------------------------------------------------
SIGNAL(TIM0_COMPB_vect)
{
	LED_OFF;
}

